@@ -2,7 +2,7 @@ import logging
 import json
 from flask import (
     flash, redirect, send_file, jsonify, make_response, url_for, session, abort)
-from ._compat import as_unicode
+from ._compat import as_unicode, string_types
 from .filemanager import uuid_originalname
 from .widgets import GroupFormListWidget, ListMasterWidget
 from .baseviews import BaseView, BaseCRUDView, BaseFormView, expose, expose_api
@@ -231,6 +231,16 @@ class RestCRUDView(BaseCRUDView):
         response.headers['Content-Type'] = "application/json"
         return response
 
+    def show_item_dict(self, item):
+        """Returns a json-able dict for show"""
+        d = {}
+        for col in self.show_columns:
+            v = getattr(item, col)
+            if not isinstance(v, (int, float, string_types)):
+                v = str(v)
+            d[col] = v
+        return d
+
     @expose_api(name='get', url='/api/get/<pk>', methods=['GET'])
     @has_access_api
     @permission_name('show')
@@ -241,15 +251,11 @@ class RestCRUDView(BaseCRUDView):
         item = self.datamodel.get(pk, self._base_filters)
         if not item:
             abort(404)
-        _item = dict()
-        for col in self.show_columns:
-            _item[col] = str(getattr(item, col))
-
         ret_json = jsonify(pk=pk,
                            label_columns=self._label_columns_json(),
                            include_columns=self.show_columns,
                            modelview_name=self.__class__.__name__,
-                           result=_item)
+                           result=self.show_item_dict(item))
         response = make_response(ret_json, 200)
         response.headers['Content-Type'] = "application/json"
         return response
@@ -258,11 +264,9 @@ class RestCRUDView(BaseCRUDView):
     @has_access_api
     @permission_name('add')
     def api_create(self):
-        is_valid_form = True
         get_filter_args(self._filters)
         exclude_cols = self._filters.get_relation_cols()
         form = self.add_form.refresh()
-
         self._fill_form_exclude_cols(exclude_cols, form)
         if form.validate():
             item = self.datamodel.obj()
@@ -273,22 +277,23 @@ class RestCRUDView(BaseCRUDView):
                 http_return_code = 200
             else:
                 http_return_code = 500
+            payload = {
+                'message': self.datamodel.message[0],
+                'item': self.show_item_dict(item),
+                'severity': self.datamodel.message[1],
+            }
         else:
-            is_valid_form = False
-        if is_valid_form:
-            response = make_response(jsonify({'message': self.datamodel.message[0],
-                                              'severity': self.datamodel.message[1]}), http_return_code)
-        else:
-            # TODO return dict with errors
-            response = make_response(jsonify({'message': 'Invalid form',
-                                              'severity': 'warning'}), 500)
-        return response
+            payload = {
+                'message': 'Validation error',
+                'error_details': form.errors,
+            }
+            http_return_code = 500
+        return make_response(jsonify(payload), http_return_code)
 
     @expose_api(name='update', url='/api/update/<pk>', methods=['PUT'])
     @has_access_api
     @permission_name('edit')
     def api_update(self, pk):
-        is_valid_form = True
         get_filter_args(self._filters)
         exclude_cols = self._filters.get_relation_cols()
 
@@ -303,25 +308,33 @@ class RestCRUDView(BaseCRUDView):
         self._fill_form_exclude_cols(exclude_cols, form)
         # trick to pass unique validation
         form._id = pk
+        http_return_code = 500
         if form.validate():
+
+            # Deleting form fields not specified as keys in POST data
+            # this allows for other Model columns to be left untouched when
+            # unspecified.
+            form_fields = set([t for t in form._fields.keys()])
+            for field in form_fields - set(request.form.keys()):
+                delattr(form, field)
+
             form.populate_obj(item)
             self.pre_update(item)
             if self.datamodel.edit(item):
                 self.post_update(item)
                 http_return_code = 200
-            else:
-                http_return_code = 500
+            payload = {
+                'message': self.datamodel.message[0],
+                'severity': self.datamodel.message[1],
+                'item': self.show_item_dict(item),
+            }
         else:
-            is_valid_form = False
-        if is_valid_form:
-            response = make_response(jsonify({'message': self.datamodel.message[0],
-                                              'severity': self.datamodel.message[1]}), http_return_code)
-        else:
-            # TODO return dict with from errors validation
-            response = make_response(jsonify({'message': 'Invalid form',
-                                              'severity': 'warning'}), 500)
-        return response
-
+            payload = {
+                'message': 'Validation error',
+                'error_details': form.errors,
+                'severity': 'warning',
+            }
+        return make_response(jsonify(payload), http_return_code)
 
     @expose_api(name='delete', url='/api/delete/<pk>', methods=['DELETE'])
     @has_access_api
@@ -437,6 +450,18 @@ class ModelView(RestCRUDView):
     def __init__(self, **kwargs):
         super(ModelView, self).__init__(**kwargs)
 
+    def post_add_redirect(self):
+        """Override this function to control the redirect after add endpoint is called."""
+        return redirect(self.get_redirect())
+
+    def post_edit_redirect(self):
+        """Override this function to control the redirect after edit endpoint is called."""
+        return redirect(self.get_redirect())
+
+    def post_delete_redirect(self):
+        """Override this function to control the redirect after edit endpoint is called."""
+        return redirect(self.get_redirect())
+
     """
     --------------------------------
             LIST
@@ -479,7 +504,7 @@ class ModelView(RestCRUDView):
     def add(self):
         widget = self._add()
         if not widget:
-            return redirect(self.get_redirect())
+            return self.post_add_redirect()
         else:
             return self.render_template(self.add_template,
                                         title=self.add_title,
@@ -496,7 +521,7 @@ class ModelView(RestCRUDView):
     def edit(self, pk):
         widgets = self._edit(pk)
         if not widgets:
-            return redirect(self.get_redirect())
+            return self.post_edit_redirect()
         else:
             return self.render_template(self.edit_template,
                                         title=self.edit_title,
@@ -513,7 +538,7 @@ class ModelView(RestCRUDView):
     @has_access
     def delete(self, pk):
         self._delete(pk)
-        return redirect(self.get_redirect())
+        return self.post_delete_redirect()
 
     @expose('/download/<string:filename>')
     @has_access
@@ -720,7 +745,10 @@ class CompactCRUDMixin(BaseCRUDView):
             return redirect(request.referrer)
         else:
             self.set_key('session_form_widget', 'add')
-            self.set_key('session_form_action', request.full_path)
+            self.set_key(
+                'session_form_action',
+                request.script_root + request.full_path
+            )
             self.set_key('session_form_title', self.add_title)
             return redirect(self.get_redirect())
 
@@ -735,7 +763,10 @@ class CompactCRUDMixin(BaseCRUDView):
             return redirect(self.get_redirect())
         else:
             self.set_key('session_form_widget', 'edit')
-            self.set_key('session_form_action', request.full_path)
+            self.set_key(
+                'session_form_action',
+                request.script_root + request.full_path
+            )
             self.set_key('session_form_title', self.add_title)
             self.set_key('session_form_edit_pk', pk)
             return redirect(self.get_redirect())
